from abjad.top.new import new
from .PitchClassSegment import PitchClassSegment


class TwelveToneRow(PitchClassSegment):
    """
    Twelve-tone row.

    ..  container:: example

        Initializes from defaults:

        >>> row = abjad.TwelveToneRow()
        >>> abjad.show(row) # doctest: +SKIP

    ..  container:: example

        Initializes from integers:

        >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
        >>> row = abjad.TwelveToneRow(numbers)
        >>> abjad.show(row) # doctest: +SKIP

    ..  container:: example

        Interpreter representation:

        >>> row
        TwelveToneRow([1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0])

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11)):
        import abjad
        assert items is not None
        PitchClassSegment.__init__(
            self,
            items=items,
            item_class=abjad.NumberedPitchClass,
            )
        self._validate_pitch_classes(self)

    ### SPECIAL METHODS ###

    def __call__(self, pitch_classes):
        r"""
        Calls row on `pitch_classes`.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Permutes pitch-classes:

            >>> row([abjad.NumberedPitchClass(2)])
            [NumberedPitchClass(9)]

            >>> row([abjad.NumberedPitchClass(3)])
            [NumberedPitchClass(3)]

            >>> row([abjad.NumberedPitchClass(4)])
            [NumberedPitchClass(6)]

        ..  container:: example

            Permutes pitch-class segment:

            >>> items = [-2, -1, 6, 7, -1, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            >>> segment_ = row(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    af'8
                    c'8
                    f'8
                    e'8
                    c'8
                    e'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Permutes row:

            >>> numbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
            >>> row_2 = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row_2) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row_2.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_3 = row(row_2)
            >>> abjad.show(row_3) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row_3.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Permutes row:

            >>> numbers = [11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
            >>> row_2 = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row_2) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row_2.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    b'8
                    bf'8
                    a'8
                    af'8
                    g'8
                    fs'8
                    f'8
                    e'8
                    ef'8
                    d'8
                    cs'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_3 = row(row_2)
            >>> abjad.show(row_3) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row_3.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    f'8
                    g'8
                    fs'8
                    ef'8
                    a'8
                    b'8
                    cs'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Permutes row:

            >>> numbers = [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]
            >>> row_2 = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row_2) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row_2.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    c'8
                    d'8
                    fs'8
                    af'8
                    g'8
                    f'8
                    ef'8
                    cs'8
                    a'8
                    e'8
                    b'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> row_3 = row(row_2)
            >>> abjad.show(row_3) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row_3.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    af'8
                    cs'8
                    a'8
                    f'8
                    bf'8
                    e'8
                    g'8
                    ef'8
                    b'8
                    d'8
                    fs'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        Returns permuted pitch-classes in object of type `pitch_classes`.
        """
        import abjad
        new_pitch_classes = []
        for pitch_class in pitch_classes:
            pitch_class = abjad.NumberedPitchClass(pitch_class)
            i = pitch_class.number
            new_pitch_class = self[i]
            new_pitch_classes.append(new_pitch_class)
        result = type(pitch_classes)(new_pitch_classes)
        return result

    def __getitem__(self, argument):
        r"""
        Gets item or slice identified by `argument`.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Gets first hexachord:

            >>> abjad.show(row[:6]) # doctest: +SKIP
            PitchClassSegment([0, 1, 11, 9, 3, 6])

            ..  docs::

                >>> lilypond_file = row[:6].__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Gets second hexachord:

            >>> abjad.show(row[-6:]) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row[-6:].__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns pitch-class segment:

            >>> row[-6:]
            PitchClassSegment([5, 4, 10, 2, 8, 0])

        """
        import abjad
        item = self._collection.__getitem__(argument)
        try:
            return PitchClassSegment(
                items=item,
                item_class=abjad.NumberedPitchClass,
                )
        except TypeError:
            return item

    def __illustrate__(self, **keywords):
        r"""
        Illustrates row.

        ..  container:: example

            Illustrates row:

            >>> row = abjad.TwelveToneRow()
            >>> abjad.show(row) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }


        ..  container:: example

            Illustrates row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = row.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns LilyPond file:

            >>> type(row.__illustrate__())
            <class 'abjad.lilypondfile.LilyPondFile.LilyPondFile'>

        """
        return super().__illustrate__(**keywords)

    def __mul__(self, argument):
        r"""
        Multiplies row by `argument`.

        ..  container:: example

            Multiplies row:

            >>> row = abjad.TwelveToneRow()
            >>> abjad.show(row) # doctest: +SKIP

            >>> segment = 2 * row
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

            >>> segment = 2 * row
            >>> abjad.show(segment) # doctest: +SKIP


            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns pitch-class segment:

            >>> segment
            PitchClassSegment([1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0, 1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0])

        Returns pitch-class segment.
        """
        return PitchClassSegment(self) * argument

    def __rmul__(self, argument):
        r"""
        Multiplies `argument` by row.

        ..  container:: example

            Multiplies integer by row:

            >>> row = abjad.TwelveToneRow()
            >>> abjad.show(row) # doctest: +SKIP

            >>> segment = row * 2
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    c'8
                    cs'8
                    d'8
                    ef'8
                    e'8
                    f'8
                    fs'8
                    g'8
                    af'8
                    a'8
                    bf'8
                    b'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies integer by row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

            >>> segment = row * 2
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns pitch-class segment:

            >>> segment
            PitchClassSegment([1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0, 1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0])

        """
        return PitchClassSegment(self) * argument

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_string(self):
        return ', '.join([str(abs(pc)) for pc in self])

    ### PRIVATE METHODS ###

    @staticmethod
    def _validate_pitch_classes(pitch_classes):
        numbers = [pc.number for pc in pitch_classes]
        numbers.sort()
        if not numbers == list(range(12)):
            message = 'must contain all twelve pitch-classes: {!r}.'
            message = message.format(pitch_classes)
            raise ValueError(message)

    ### PUBLIC PROPERTIES ###

    @property
    def item_class(self):
        """
        Gets item class of row.

        ..  container:: example

            Gets item class:

            >>> row = abjad.TwelveToneRow()
            >>> abjad.show(row) # doctest: +SKIP

            >>> row.item_class
            <class 'abjad.pitch.NumberedPitchClass.NumberedPitchClass'>

        ..  container:: example

            Gets item class:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

            >>> row.item_class
            <class 'abjad.pitch.NumberedPitchClass.NumberedPitchClass'>

        ..  container:: example

            Returns numbered pitch-class class:

            >>> type(row.item_class)
            <class 'abc.ABCMeta'>

        """
        return super().item_class

    @property
    def items(self):
        """
        Gets items in row.

        ..  container:: example

            Gets items in row:

            >>> row = abjad.TwelveToneRow()
            >>> abjad.show(row) # doctest: +SKIP


            >>> for item in row.items:
            ...     item
            ...
            NumberedPitchClass(0)
            NumberedPitchClass(1)
            NumberedPitchClass(2)
            NumberedPitchClass(3)
            NumberedPitchClass(4)
            NumberedPitchClass(5)
            NumberedPitchClass(6)
            NumberedPitchClass(7)
            NumberedPitchClass(8)
            NumberedPitchClass(9)
            NumberedPitchClass(10)
            NumberedPitchClass(11)

        ..  container:: example

            Gets items in row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

            >>> for item in row.items:
            ...     item
            ...
            NumberedPitchClass(1)
            NumberedPitchClass(11)
            NumberedPitchClass(9)
            NumberedPitchClass(3)
            NumberedPitchClass(6)
            NumberedPitchClass(7)
            NumberedPitchClass(5)
            NumberedPitchClass(4)
            NumberedPitchClass(10)
            NumberedPitchClass(2)
            NumberedPitchClass(8)
            NumberedPitchClass(0)

        ..  container:: example

            Returns list:

            >>> isinstance(row.items, list)
            True

        """
        return super().items

    ### PUBLIC METHODS ###

    def count(self, item):
        """
        Counts `item` in row.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Counts pitch-class 11 in row:

            >>> row.count(11)
            1

        ..  container:: example

            Counts pitch-class 9 in row:

            >>> row.count(9)
            1

        ..  container:: example

            Counts string in row:

            >>> row.count('text')
            0

        ..  container:: example

            Returns nonnegative integer equal to 0 or 1:

            >>> isinstance(row.count('text'), int)
            True

        """
        return super().count(item)

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes row from `selection`.

        Not yet implemented.

        Returns twelve-tone row.
        """
        raise NotImplementedError

    def has_duplicates(self):
        """
        Is false for all rows.

        ..  container:: example

            Is false:

            >>> row = abjad.TwelveToneRow()
            >>> abjad.show(row) # doctest: +SKIP

            >>> row.has_duplicates()
            False

        ..  container:: example

            Is false:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

            >>> row.has_duplicates()
            False

        Twelve-tone rows have no duplicates.

        Returns false.
        """
        return super().has_duplicates()

    def index(self, item):
        """
        Gets index of `item` in row.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Gets index of pitch-class 11:

            >>> row.index(11)
            1

        ..  container:: example

            Gets index of pitch-class 9:

            >>> row.index(9)
            2

        ..  container:: example

            Returns nonnegative integer less than 12:

            >>> isinstance(row.index(9), int)
            True

        """
        return super().index(item)

    def invert(self, axis=None):
        r"""
        Inverts row about optional `axis`.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Inverts row about first pitch-class when `axis` is none:

            >>> inversion = row.invert()
            >>> abjad.show(inversion) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = inversion.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    ef'8
                    f'8
                    b'8
                    af'8
                    g'8
                    a'8
                    bf'8
                    e'8
                    c'8
                    fs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            First pitch-classes are equal:

            >>> row[0] == inversion[0]
            True

        ..  container:: example

            Inverts row about pitch-class 1:

            >>> inversion = row.invert(axis=1)
            >>> abjad.show(inversion) # doctest: +SKIP
            TwelveToneRow([1, 3, 5, 11, 8, 7, 9, 10, 4, 0, 6, 2])

            ..  docs::

                >>> lilypond_file = inversion.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    ef'8
                    f'8
                    b'8
                    af'8
                    g'8
                    a'8
                    bf'8
                    e'8
                    c'8
                    fs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            Same result as above.

        ..  container:: example

            Inverts row about pitch-class 0:

            >>> inversion = row.invert(axis=0)
            >>> abjad.show(inversion) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = inversion.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    b'8
                    cs'8
                    ef'8
                    a'8
                    fs'8
                    f'8
                    g'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Inverts row about pitch-class 5:

            >>> inversion = row.invert(axis=5)
            >>> abjad.show(inversion) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = inversion.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    a'8
                    b'8
                    cs'8
                    g'8
                    e'8
                    ef'8
                    f'8
                    fs'8
                    c'8
                    af'8
                    d'8
                    bf'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns twelve-tone row:

            >>> inversion
            TwelveToneRow([9, 11, 1, 7, 4, 3, 5, 6, 0, 8, 2, 10])

        """
        if axis is None:
            axis = self[0]
        items = [pc.invert(axis=axis) for pc in self]
        return new(self, items=items)

    def multiply(self, n=1):
        r"""
        Multiplies pitch-classes in row by `n`.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Multiplies pitch-classes in row by 5:

            >>> multiplication = row.multiply(n=5)
            >>> abjad.show(multiplication) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = multiplication.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    f'8
                    g'8
                    a'8
                    ef'8
                    fs'8
                    b'8
                    cs'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies pitch-classes in row by 7:

            >>> multiplication = row.multiply(n=7)
            >>> abjad.show(multiplication) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = multiplication.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    g'8
                    f'8
                    ef'8
                    a'8
                    fs'8
                    cs'8
                    b'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Multiplies pitch-classes in row by 1:

            >>> multiplication = row.multiply(n=1)
            >>> abjad.show(multiplication) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = multiplication.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns twelve-tone row:

            >>> multiplication
            TwelveToneRow([1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0])

        """
        return super().multiply(n=n)

    def retrograde(self):
        r"""
        Gets retrograde of row.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Gets retrograde of row:

            >>> retrograde = row.retrograde()
            >>> abjad.show(retrograde) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = retrograde.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    af'8
                    d'8
                    bf'8
                    e'8
                    f'8
                    g'8
                    fs'8
                    ef'8
                    a'8
                    b'8
                    cs'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Gets retrograde of retrograde of row:

            >>> retrograde = row.retrograde().retrograde()
            >>> abjad.show(retrograde) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = retrograde.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> retrograde == row
            True

        ..  container:: example

            Returns row:

            >>> retrograde
            TwelveToneRow([1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0])

        """
        return super().retrograde()

    def rotate(self, n=0, stravinsky=False):
        r"""
        Rotates row by index `n`.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Rotates row to the right:

            >>> rotation = row.rotate(n=1)
            >>> abjad.show(rotation) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = rotation.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Rotates row to the left:

            >>> rotation = row.rotate(n=-1)
            >>> abjad.show(rotation) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = rotation.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    cs'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Rotates row by zero:

            >>> rotation = row.rotate(n=0)
            >>> abjad.show(rotation) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = rotation.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> rotation == row
            True

        ..  container:: example

            Stravinsky-style rotation back-transposes row to zero:

            >>> rotation = row.rotate(n=-1, stravinsky=True)
            >>> abjad.show(rotation) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = rotation.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    bf'8
                    e'8
                    g'8
                    af'8
                    fs'8
                    f'8
                    b'8
                    ef'8
                    a'8
                    cs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns row:

            >>> rotation
            TwelveToneRow([0, 10, 4, 7, 8, 6, 5, 11, 3, 9, 1, 2])

        """
        return super().rotate(n=n, stravinsky=stravinsky)

    def transpose(self, n=0):
        r"""
        Transposes row by index `n`.

        ..  container:: example

            Example row:

            >>> numbers = [1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0]
            >>> row = abjad.TwelveToneRow(numbers)
            >>> abjad.show(row) # doctest: +SKIP

        ..  container:: example

            Transposes row by positive index:

            >>> transposition = row.transpose(n=13)
            >>> abjad.show(transposition) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = transposition.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    d'8
                    c'8
                    bf'8
                    e'8
                    g'8
                    af'8
                    fs'8
                    f'8
                    b'8
                    ef'8
                    a'8
                    cs'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Transposes row by negative index:

            >>> transposition = row.transpose(n=-13)
            >>> abjad.show(transposition) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = transposition.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    bf'8
                    af'8
                    d'8
                    f'8
                    fs'8
                    e'8
                    ef'8
                    a'8
                    cs'8
                    g'8
                    b'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Transposes row by zero index:

            >>> transposition = row.transpose(n=0)
            >>> abjad.show(transposition) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = transposition.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    cs'8
                    b'8
                    a'8
                    ef'8
                    fs'8
                    g'8
                    f'8
                    e'8
                    bf'8
                    d'8
                    af'8
                    c'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> transposition == row
            True

        ..  container:: example

            Returns row:

            >>> transposition
            TwelveToneRow([1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8, 0])

        """
        return super().transpose(n=n)



def sign(n):
    """
    Gets sign of ``n``.

    ..  container:: example

        Returns -1 on negative ``n``:

        >>> abjad.mathtools.sign(-96.2)
        -1

        Returns 0 when ``n`` is 0:

        >>> abjad.mathtools.sign(0)
        0

        Returns 1 on positive ``n``:

        >>> abjad.mathtools.sign(abjad.Duration(9, 8))
        1

    Returns -1, 0 or 1.
    """
    return (0 < n) - (n < 0)



def timespan_2_delays_timespan_1(timespan_1=None, timespan_2=None, hold=False):
    """
    Makes time relation indicating that ``timespan_2`` delays ``timespan_1``.

    ..  container:: example

        >>> relation = abjad.timespans.timespan_2_delays_timespan_1()
        >>> abjad.f(relation)
        abjad.timespans.TimespanTimespanTimeRelation(
            inequality=abjad.timespans.CompoundInequality(
                [
                    abjad.TimespanInequality('timespan_2.start_offset <= timespan_1.start_offset'),
                    abjad.TimespanInequality('timespan_1.start_offset < timespan_2.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    Returns time relation or boolean.
    """
    from abjad import timespans

    inequality = timespans.CompoundInequality([
        'timespan_2.start_offset <= timespan_1.start_offset',
        'timespan_1.start_offset < timespan_2.stop_offset',
        ])

    time_relation = timespans.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



import collections
from .TypedCollection import TypedCollection


class TypedCounter(TypedCollection, collections.MutableMapping):
    """
    Typed counter.

    ..  container:: example

        >>> counter = abjad.TypedCounter(
        ...     [0, "c'", 1, True, "cs'", "df'"],
        ...     item_class=abjad.NumberedPitch,
        ...     )

        >>> abjad.f(counter)
        abjad.TypedCounter(
            {
                abjad.NumberedPitch(0): 2,
                abjad.NumberedPitch(1): 4,
                },
            item_class=abjad.NumberedPitch,
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        **keywords
        ):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        self._collection = collections.Counter()
        self.update(items, **keywords)

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        """
        Adds typed counter to ``argument``.

        Returns new typed counter.
        """
        if (not isinstance(argument, type(self)) or
            not self.item_class == argument.item_class):
            return NotImplemented
        result = type(self)()
        result._collection = self._collection + argument._collection
        return result

    def __and__(self, argument):
        """
        Logical AND of typed counter and ``argument``.

        Returns new typed counter.
        """
        if (not isinstance(argument, type(self)) or
            not self.item_class == argument.item_class):
            return NotImplemented
        result = type(self)()
        result._collection = self._collection & argument._collection
        return result

    def __delitem__(self, item):
        """
        Deletes ``item`` from typed counter.

        Returns none.
        """
        item = self._item_coercer(item)
        if item in self._collection:
            dict.__delitem__(self._collection, item)

    def __getitem__(self, argument):
        """
        Gets item or slice identified by ``argument``.

        Returns item or slice.
        """
        argument = self._item_coercer(argument)
        return self._collection.__getitem__(argument)

    def __or__(self, argument):
        """
        Logical OR of typed counter and ``argument``.

        Returns new typed counter.
        """
        if (not isinstance(argument, type(self)) or
            not self.item_class == argument.item_class):
            return NotImplemented
        result = type(self)()
        result._collection = self._collection | argument._collection
        return result

    def __radd__(self, argument):
        """
        Adds ``argument`` to typed counter.

        Returns new typed counter.
        """
        if (not isinstance(argument, type(self)) or
            not self.item_class == argument.item_class):
            return NotImplemented
        result = type(self)()
        result._collection = argument._collection + self._collection
        return result

    def __reduce__(self):
        """
        Reduces typed counter.

        Returns new typed counter.
        """
        return type(self), (dict(self._collection),)

    def __setitem__(self, item, value):
        """
        Sets typed counter ``item`` to ``value``.

        Returns none.
        """
        item = self._item_coercer(item)
        self._collection.__setitem__(item, value)

    def __sub__(self, argument):
        """
        Subtracts ``argument`` from typed counter.

        Returns new typed counter.
        """
        if (not isinstance(argument, type(self)) or
            not self.item_class == argument.item_class):
            return NotImplemented
        result = type(self)()
        result._collection = self._collection - argument._collection
        return result

    ### PRIVATE METHODS ###

    def _coerce_arguments(self, items=None, **keywords):
        def _coerce_mapping(items):
            the_items = {}
            for item, count in items.items():
                item = self._item_coercer(item)
                if item not in the_items:
                    the_items[item] = 0
                the_items[item] += count
            return the_items
        the_items = []
        if items is not None:
            if isinstance(items, collections.Mapping):
                items = _coerce_mapping(items)
            else:
                the_items = []
                for item in items:
                    the_items.append(self._item_coercer(item))
        itemdict = _coerce_mapping(keywords)
        return the_items, itemdict

    def _get_format_specification(self):
        import abjad
        agent = abjad.StorageFormatManager(self)
        names = list(agent.signature_keyword_names)
        if 'items' in names:
            names.remove('items')
        return abjad.FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=[self._collection],
            storage_format_kwargs_names=names,
            template_names=names,
            )

    ### PUBLIC METHODS ###

    def clear(self):
        """
        Clears typed counter.

        Returns none.
        """
        self._collection.clear()

    def copy(self):
        """
        Copies typed counter.

        Returns new typed counter.
        """
        return type(self)(self)

    def elements(self):
        """
        Elements in typed counter.
        """
        return self._collection.elements()

    @classmethod
    def fromkeys(class_, iterable, v=None):
        """
        Makes new typed counter from ``iterable``.

        Not yet impelemented.

        Will return new typed counter.
        """
        message = '{}.fromkeys() is undefined. Use {}(iterable) instead.'
        message = message.format(class_.__name__, class_.__name__)
        raise NotImplementedError(message)

    def items(self):
        """
        Items in typed counter.

        Returns tuple.
        """
        return list(self._collection.items())

    def keys(self):
        """
        Iterates keys in typed counter.
        """
        return iter(self._collection.keys())

    def most_common(self, n=None):
        """
        Please document.
        """
        return self._collection(n=n)

    def subtract(self, iterable=None, **keywords):
        """
        Subtracts ``iterable`` from typed counter.
        """
        items, itemdict = self._coerce_arguments(iterable, **keywords)
        self._collection.subtract(items, **itemdict)

    def update(self, iterable=None, **keywords):
        """
        Updates typed counter with ``iterable``.
        """
        items, itemdict = self._coerce_arguments(iterable, **keywords)
        self._collection.update(items, **itemdict)

    def values(self):
        """
        Iterates values in typed counter.
        """
        return iter(self._collection.values())

    def viewitems(self):
        """
        Please document.
        """
        return self._collection.items()

    def viewkeys(self):
        """
        Please document.
        """
        return self._collection.keys()

    def viewvalues(self):
        """
        Please document.
        """
        return self._collection.values()



"""
Abjad's system internals.
"""

from .AbjadConfiguration import AbjadConfiguration
from .AbjadObject import AbjadObject
from .AbjadValueObject import AbjadValueObject
from .BenchmarkScoreMaker import BenchmarkScoreMaker
from .Configuration import Configuration
from .ContextManager import ContextManager
from .FilesystemState import FilesystemState
from .ForbidUpdate import ForbidUpdate
from .FormatSpecification import FormatSpecification
from .IOManager import IOManager
from .ImportManager import ImportManager
from .LilyPondFormatBundle import LilyPondFormatBundle
from .LilyPondFormatManager import LilyPondFormatManager
from .NullContextManager import NullContextManager
from .Parser import Parser
from .PersistenceManager import PersistenceManager
from .ProgressIndicator import ProgressIndicator
from .RedirectedStreams import RedirectedStreams
from .Signature import Signature
from .SlotContributions import SlotContributions
from .StorageFormatManager import StorageFormatManager
from .StorageFormatSpecification import StorageFormatSpecification
from .Tag import Tag
from .Tags import Tags
from .TemporaryDirectory import TemporaryDirectory
from .TemporaryDirectoryChange import TemporaryDirectoryChange
from .TestManager import TestManager
from .Timer import Timer
from .UpdateManager import UpdateManager
from .WellformednessManager import WellformednessManager
from .Wrapper import Wrapper



from .Inequality import Inequality


class LengthInequality(Inequality):
    """
    Length inequality.

    ..  container:: example

        >>> inequality = abjad.LengthInequality('<', 4)
        >>> abjad.f(inequality)
        abjad.LengthInequality(
            operator_string='<',
            length=4,
            )

        >>> inequality([1, 2, 3])
        True

        >>> inequality([1, 2, 3, 4])
        False

        >>> inequality([1, 2, 3, 4, 5])
        False

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Inequalities'

    __slots__ = (
        '_length',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        operator_string='<',
        length=None,
        ):
        import abjad
        Inequality.__init__(self, operator_string=operator_string)
        if length is None:
            length = abjad.mathtools.Infinity()
        assert 0 <= length
        infinities = (
            abjad.mathtools.Infinity(),
            abjad.mathtools.NegativeInfinity(),
            )
        if length not in infinities:
            length = int(length)
        self._length = length

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        """
        Calls inequality on ``argument``.

        Returns true or false.
        """
        return self._operator_function(len(argument), self.length)

    ### PUBLIC PROPERTIES ###

    @property
    def length(self):
        """
        Gets length.

        Returns integer.
        """
        return self._length



import copy
import typing
from abjad.system.LilyPondFormatManager import LilyPondFormatManager
from abjad.utilities.String import String
from .LilyPondNameManager import LilyPondNameManager


class LilyPondTweakManager(LilyPondNameManager):
    """
    LilyPond tweak manager.

    ..  container:: example

        Tweak managers are created by the ``abjad.tweak()`` factory function:

        >>> beam = abjad.Beam()
        >>> abjad.tweak(beam)
        LilyPondTweakManager()

        Set an attribute like this:

        >>> abjad.tweak(beam).color = 'red'

        The state of the tweak manager has changed:

        >>> abjad.tweak(beam)
        LilyPondTweakManager(('color', 'red'))

        And the value of the attribute just set is available like this:

        >>> abjad.tweak(beam).color
        'red'

        Trying to get an attribute that has not yet been set raises an
        attribute error:

        >>> abjad.tweak(beam).foo
        Traceback (most recent call last):
            ...
        AttributeError: LilyPondTweakManager object has no attribute 'foo'.
        
    """

    ### SPECIAL METHODS ###

    def __getattr__(self, name) -> typing.Union[
        LilyPondNameManager, typing.Any,
        ]:
        r"""
        Gets LilyPondNameManager (or LilyPondGrobNameManager) keyed to 
        ``name``.

        ..  container:: example

            Getting a grob name returns a LilyPondNameManager:

            >>> hairpin = abjad.Hairpin('p < f')
            >>> abjad.tweak(hairpin).dynamic_line_spanner
            LilyPondNameManager()

            Set a tweak with explicit grob like this:

            >>> abjad.tweak(hairpin).dynamic_line_spanner.staff_padding = 5

            This changes tweak manager state:

            >>> abjad.tweak(hairpin)
            LilyPondTweakManager(('dynamic_line_spanner', LilyPondNameManager(('staff_padding', 5))))

            Grob is available like this:

            >>> abjad.tweak(hairpin).dynamic_line_spanner
            LilyPondNameManager(('staff_padding', 5))

            Attribute is available like this:

            >>> abjad.tweak(hairpin).dynamic_line_spanner.staff_padding
            5

            Grob-explicit tweak appears like this:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(hairpin, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP
            
            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \tweak DynamicLineSpanner.staff-padding #5
                    \<
                    \p
                    d'4
                    e'4
                    f'4
                    \f
                }

        ..  container:: example

            Tweak expressions work like this:

            >>> abjad.tweak('red').color
            LilyPondTweakManager(('color', 'red'))

            >>> abjad.tweak(6).Y_offset
            LilyPondTweakManager(('Y_offset', 6))

            >>> abjad.tweak(False).bound_details__left_broken__text
            LilyPondTweakManager(('bound_details__left_broken__text', False))

        """
        from abjad.ly import contexts
        from abjad.ly import grob_interfaces
        if '_pending_value' in vars(self):
            _pending_value = self._pending_value
            self.__setattr__(name, _pending_value)
            delattr(self, '_pending_value')
            return self
        camel_name = String(name).to_upper_camel_case()
        if name.startswith('_'):
            try:
                return vars(self)[name]
            except KeyError:
                type_name = type(self).__name__
                message = f'{type_name} object has no attribute {name!r}.'
                raise AttributeError(message)
        elif camel_name in grob_interfaces:
            try:
                return vars(self)[name]
            except KeyError:
                vars(self)[name] = LilyPondNameManager()
                return vars(self)[name]
        else:
            try:
                return vars(self)[name]
            except KeyError:
                type_name = type(self).__name__
                message = f'{type_name} object has no attribute {name!r}.'
                raise AttributeError(message)

    ### PRIVATE METHODS ###

    def _get_attribute_tuples(self) -> typing.List[typing.Tuple]:
        result: typing.List[typing.Tuple] = []
        for name, value in vars(self).items():
            if type(value) is LilyPondNameManager:
                grob_name = name
                grob_proxy = value
                pairs = iter(vars(grob_proxy).items())
                for attribute_name, attribute_value in pairs:
                    triple = (grob_name, attribute_name, attribute_value)
                    result.append(triple)
            else:
                attribute_name = name
                attribute_value = value
                pair = (attribute_name, attribute_value)
                result.append(pair)
        return result

    def _list_format_contributions(self, directed=True):
        result = []
        for attribute_tuple in self._get_attribute_tuples():
            if len(attribute_tuple) == 2:
                grob = None
                attribute = attribute_tuple[0]
                value = attribute_tuple[1]
            elif len(attribute_tuple) == 3:
                grob = attribute_tuple[0]
                attribute = attribute_tuple[1]
                value = attribute_tuple[2]
            else:
                message = 'invalid attribute tuple: {attribute_tuple!r}.'
                raise ValueError(message)
            string = LilyPondFormatManager.make_lilypond_tweak_string(
                attribute,
                value,
                directed=directed,
                grob=grob,
                )
            result.append(string)
        result.sort()
        return result

    ### PUBLIC METHODS ###

    @staticmethod
    def set_tweaks(
        argument,
        tweaks: typing.Union[
            typing.List[typing.Tuple],
            'LilyPondTweakManager',
            None,
            ],
        ) -> None:
        """
        Sets ``tweaks`` on ``argument``.
        """
        if not hasattr(argument, '_tweaks'):
            name = type(argument).__name__
            raise NotImplementedError(f'{name} does not allow tweaks (yet).')
        if not tweaks:
            return
        if isinstance(tweaks, LilyPondTweakManager):
            tweaks = tweaks._get_attribute_tuples()
        if not isinstance(tweaks, list):
            raise Exception(f'tweaks must be list of tuples (not {tweaks!r}).')
        assert all(isinstance(_, tuple) for _ in tweaks), repr(tweaks)
        if not tweaks:
            return
        if argument._tweaks is None:
            argument._tweaks = LilyPondTweakManager()
        manager = argument._tweaks
        for tweak in tweaks:
            if len(tweak) == 2:
                attribute, value = tweak
                value = copy.copy(value)
                setattr(manager, attribute, value)
            elif len(tweak) == 3:
                grob, attribute, value = tweak
                value = copy.copy(value)
                grob = getattr(manager, grob)
                setattr(grob, attribute, value)
            else:
                message = 'tweak tuple must have length 2 or 3'
                message += f' (not {tweak!r}).'
                raise ValueError(message)



from abjad import mathtools
from abjad.system.FormatSpecification import FormatSpecification
from . import constants
from .Interval import Interval


class NamedInterval(Interval):
    """
    Named interval.

    ..  container:: example

        Initializes ascending major ninth from string:

        >>> abjad.NamedInterval('+M9')
        NamedInterval('+M9')

    ..  container:: example

        Initializes descending major third from number of semitones:

        >>> abjad.NamedInterval(-4)
        NamedInterval('-M3')

    ..  container:: example

        Initializes from other named interval:

        >>> abjad.NamedInterval(abjad.NamedInterval(-4))
        NamedInterval('-M3')

    ..  container:: example

        Initializes from numbered interval:

        >>> abjad.NamedInterval(abjad.NumberedInterval(3))
        NamedInterval('+m3')

    ..  container:: example

        Initializes from pair of quality and diatonic number:

        >>> abjad.NamedInterval(('M', 3))
        NamedInterval('+M3')

    """

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, name='P1'):
        super().__init__(name or 'P1')

    ### SPECIAL METHODS ###

    def __abs__(self):
        """
        Gets absolute value of named interval.

        ..  container:: example

            >>> abs(abjad.NamedInterval('+M9'))
            NamedInterval('+M9')

            >>> abs(abjad.NamedInterval('-M9'))
            NamedInterval('+M9')

        Returns named interval.
        """
        return type(self)((
            self.quality,
            abs(self.number),
            ))

    def __add__(self, argument):
        """
        Adds `argument` to named interval.

        ..  container:: example

            >>> abjad.NamedInterval('M9') + abjad.NamedInterval('M2')
            NamedInterval('+M10')

        Returns new named interval.
        """
        import abjad
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        dummy_pitch = abjad.NamedPitch(0)
        new_pitch = dummy_pitch + self + argument
        return NamedInterval.from_pitch_carriers(dummy_pitch, new_pitch)

    def __copy__(self, *arguments):
        """
        Copies named interval.

        >>> import copy

        ..  container:: example

            >>> copy.copy(abjad.NamedInterval('+M9'))
            NamedInterval('+M9')

        Returns new named interval.
        """
        return type(self)((
            self.quality,
            self.number,
            ))

    def __eq__(self, argument):
        """
        Is true when named interval equal `argument`.

        ..  container:: example

            >>> interval_1 = abjad.NamedInterval('m2')
            >>> interval_2 = abjad.NamedInterval('m2')
            >>> interval_3 = abjad.NamedInterval('m9')

            >>> interval_1 == interval_1
            True
            >>> interval_1 == interval_2
            True
            >>> interval_1 == interval_3
            False

            >>> interval_2 == interval_1
            True
            >>> interval_2 == interval_2
            True
            >>> interval_2 == interval_3
            False

            >>> interval_3 == interval_1
            False
            >>> interval_3 == interval_2
            False
            >>> interval_3 == interval_3
            True

        """
        return super().__eq__(argument)

    def __float__(self):
        """
        Coerce to semitones as float.

        Returns float.
        """
        return float(self.semitones)

    def __hash__(self):
        """
        Hashes named interval.

        Returns number.
        """
        return super().__hash__()

    def __lt__(self, argument):
        """
        Is true when `argument` is a named interval with a number greater
        than that of this named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9') < abjad.NamedInterval('+M10')
            True

        ..  container:: example

            Also true when `argument` is a named interval with a
            number equal to this named interval and with semitones greater than
            this named interval:

            >>> abjad.NamedInterval('+m9') < abjad.NamedInterval('+M9')
            True

        ..  container:: example

            Otherwise false:

            >>> abjad.NamedInterval('+M9') < abjad.NamedInterval('+M2')
            False

        Returns true or false.
        """
        if isinstance(argument, type(self)):
            if self.number == argument.number:
                return self.semitones < argument.semitones
            return self.number < argument.number
        return False

    def __mul__(self, argument):
        """
        Multiplies named interval by `argument`.

        ..  container:: example

            >>> 3 * abjad.NamedInterval('+M9')
            NamedInterval('+A25')

        Returns new named interval.
        """
        import abjad
        if not isinstance(argument, int):
            message = 'must be integer: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        dummy_pitch = abjad.NamedPitch(0)
        for i in range(abs(argument)):
            dummy_pitch += self
        result = NamedInterval.from_pitch_carriers(
            abjad.NamedPitch(0),
            dummy_pitch,
            )
        if argument < 0:
            return -result
        return result

    def __neg__(self):
        """
        Negates named interval.

        ..  container:: example

            >>> -abjad.NamedInterval('+M9')
            NamedInterval('-M9')

        ..  container:: example

            >>> -abjad.NamedInterval('-M9')
            NamedInterval('+M9')

        Returns new named interval.
        """
        return type(self)((
            self.quality,
            -self.number,
            ))

    def __radd__(self, argument):
        """
        Adds named interval to `argument`.

        ..  container:: example

            >>> abjad.NamedInterval('M9') + abjad.NamedInterval('M2')
            NamedInterval('+M10')

        Returns new named interval.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return argument.__add__(self)

    def __rmul__(self, argument):
        """
        Multiplies `argument` by named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9') * 3
            NamedInterval('+A25')

        Returns new named interval.
        """
        return self * argument

    def __str__(self):
        """
        Gets string representation of named interval.

        ..  container:: example

            >>> str(abjad.NamedInterval('+M9'))
            '+M9'

        Returns string.
        """
        return self.name

    def __sub__(self, argument):
        """
        Subtracts `argument` from named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9') - abjad.NamedInterval('+M2')
            NamedInterval('+P8')

            >>> abjad.NamedInterval('+M2') - abjad.NamedInterval('+M9')
            NamedInterval('-P8')

        Returns new named interval.
        """
        import abjad
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        dummy_pitch = abjad.NamedPitch(0)
        new_pitch = dummy_pitch + self - argument
        return NamedInterval.from_pitch_carriers(dummy_pitch, new_pitch)

    ### PRIVATE PROPERTIES ###

    def _from_named_parts(self, direction, quality, diatonic_number):
        import abjad
        octaves = 0
        diatonic_pc_number = abs(diatonic_number)
        while diatonic_pc_number > 7:
            octaves += 1
            diatonic_pc_number -= 7
        if diatonic_pc_number == 1 and quality == 'P' and diatonic_number >= 8:
            octaves -= 1
            diatonic_pc_number = 8
        self._octaves = octaves
        if direction:
            diatonic_pc_number *= direction
        self._interval_class = abjad.NamedIntervalClass((
            quality,
            diatonic_pc_number,
            ))

    def _from_number(self, argument):
        direction, quality, diatonic_number = self._numbered_to_named(argument)
        self._from_named_parts(direction, quality, diatonic_number)

    def _from_interval_or_interval_class(self, argument):
        try:
            quality = argument.quality
            diatonic_number = abs(argument.number)
            direction = mathtools.sign(argument.number)
        except AttributeError:
            direction, quality, diatonic_number = self._numbered_to_named(argument)
        self._from_named_parts(direction, quality, diatonic_number)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.name]
        return FormatSpecification(
            client=self,
            coerce_for_equality=True,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        """
        Gets direction number of named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9').direction_number
            1

            >>> abjad.NamedInterval('+dim2').direction_number
            1

            >>> abjad.NamedInterval('+A1').direction_number
            1

            >>> abjad.NamedInterval('P1').direction_number
            0

            >>> abjad.NamedInterval('-m3').direction_number
            -1

        Returns ``-1``, ``0`` or ``1``.
        """
        if self.quality == 'P' and abs(self.number) == 1:
            return 0
        return mathtools.sign(self.number)

    @property
    def interval_class(self):
        """
        Gets interval class of named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9').interval_class
            NamedIntervalClass('+M2')

            >>> abjad.NamedInterval('-M9').interval_class
            NamedIntervalClass('-M2')

            >>> abjad.NamedInterval('P1').interval_class
            NamedIntervalClass('P1')

            >>> abjad.NamedInterval('+P8').interval_class
            NamedIntervalClass('+P8')

        Returns named interval-class.
        """
        return self._interval_class

    @property
    def name(self):
        """
        Gets name of named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9').name
            '+M9'

        Returns string.
        """
        direction_symbol = constants._direction_number_to_direction_symbol[
            self.direction_number]
        return '{}{}{}'.format(
            direction_symbol,
            self._interval_class.quality,
            abs(self.number),
            )

    @property
    def number(self):
        """
        Gets number of named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9').number
            9

        Returns nonnegative number.
        """
        number = self._interval_class._number
        direction = mathtools.sign(number)
        number = abs(number) + (7 * self.octaves)
        return number * direction

    @property
    def octaves(self):
        """
        Gets octaves of interval.

        Returns nonnegative number.
        """
        return self._octaves

    @property
    def quality(self):
        """
        Gets quality of named interval.

        Returns string.
        """
        return self._interval_class.quality

    @property
    def semitones(self):
        """
        Gets semitones of named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9').semitones
            14

            >>> abjad.NamedInterval('-M9').semitones
            -14

            >>> abjad.NamedInterval('P1').semitones
            0

            >>> abjad.NamedInterval('+P8').semitones
            12

            >>> abjad.NamedInterval('-P8').semitones
            -12

        Returns number.
        """
        direction = self.direction_number
        diatonic_number = abs(self._interval_class._number)
        quality = self._validate_quality_and_diatonic_number(
            self.quality, diatonic_number,
        )
        diatonic_number += 7 * self._octaves
        return self._named_to_numbered(direction, quality, diatonic_number)

    @property
    def staff_spaces(self):
        """
        Gets staff spaces of named interval.

        ..  container:: example

            >>> abjad.NamedInterval('+M9').staff_spaces
            8

            >>> abjad.NamedInterval('-M9').staff_spaces
            -8

            >>> abjad.NamedInterval('P1').staff_spaces
            0

            >>> abjad.NamedInterval('+P8').staff_spaces
            7

            >>> abjad.NamedInterval('-P8').staff_spaces
            -7

        Returns nonnegative integer.
        """
        if self.direction_number == -1:
            return self.number + 1
        elif not self.direction_number:
            return 0
        elif self.direction_number == 1:
            return self.number - 1

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        """
        Makes named interval calculated from `pitch_carrier_1` to
        `pitch_carrier_2`.

        ..  container:: example

            >>> abjad.NamedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch(-2),
            ...     abjad.NamedPitch(12),
            ...     )
            NamedInterval('+M9')

            >>> abjad.NamedInterval.from_pitch_carriers(
            ...     abjad.NamedPitch("css'"),
            ...     abjad.NamedPitch("cff'"),
            ...     )
            NamedInterval('-AAAA1')

            >>> abjad.NamedInterval.from_pitch_carriers("c'", "cs'''")
            NamedInterval('+A15')

            >>> abjad.NamedInterval.from_pitch_carriers('c', 'cqs')
            NamedInterval('+P+1')

            >>> abjad.NamedInterval.from_pitch_carriers("cf'", 'bs')
            NamedInterval('-dd2')

            >>> abjad.NamedInterval.from_pitch_carriers("cff'", 'aqs')
            NamedInterval('-ddd+3')

            >>> abjad.NamedInterval.from_pitch_carriers("cff'", 'atqs')
            NamedInterval('-dddd+3')

        Returns named interval.
        """
        import abjad

        pitch_1 = abjad.NamedPitch(pitch_carrier_1)
        pitch_2 = abjad.NamedPitch(pitch_carrier_2)
        degree_1 = pitch_1._get_diatonic_pitch_number()
        degree_2 = pitch_2._get_diatonic_pitch_number()
        named_sign = mathtools.sign(degree_1 - degree_2)
        named_i_number = abs(degree_1 - degree_2) + 1
        numbered_sign = mathtools.sign(
            float(abjad.NumberedPitch(pitch_1)) -
            float(abjad.NumberedPitch(pitch_2))
            )
        numbered_i_number = abs(
            float(abjad.NumberedPitch(pitch_1)) -
            float(abjad.NumberedPitch(pitch_2))
            )
        named_ic_number = named_i_number
        numbered_ic_number = numbered_i_number

        while named_ic_number > 8 and numbered_ic_number > 12:
            named_ic_number -= 7
            numbered_ic_number -= 12

        # Multiply-diminished intervals can have opposite signs
        if named_sign and (named_sign == -numbered_sign):
            numbered_ic_number *= -1

        quartertone = ''
        if numbered_ic_number % 1:
            quartertone = '+'
            numbered_ic_number -= 0.5

        mapping = {
            value: key
            for key, value in
            constants._diatonic_number_and_quality_to_semitones[
                named_ic_number].items()
            }

        quality = ''

        while numbered_ic_number > max(mapping):
            numbered_ic_number -= 1
            quality += 'A'

        while numbered_ic_number < min(mapping):
            numbered_ic_number += 1
            quality += 'd'

        quality += mapping[numbered_ic_number]
        quality += quartertone
        direction = 1
        if pitch_2 < pitch_1:
            direction = -1

        return class_((quality, named_i_number * direction))

    def transpose(self, pitch_carrier):
        """
        Transposes `pitch_carrier` by named interval.

        ..  container:: example

            Transposes chord:

            >>> chord = abjad.Chord("<c' e' g'>4")

            >>> interval = abjad.NamedInterval('+m2')
            >>> interval.transpose(chord)
            Chord("<df' f' af'>4")

        Returns new (copied) object of `pitch_carrier` type.
        """
        return super().transpose(pitch_carrier)



from .Vector import Vector


class PitchClassVector(Vector):
    """
    Pitch-class vector.

    ..  container:: example

        Pitch-class vector:

        >>> vector = abjad.PitchClassVector(
        ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
        ...     item_class=abjad.NumberedPitchClass,
        ...     )

        >>> items = sorted(vector.items())
        >>> for pitch_class, count in items:
        ...     print(pitch_class, count)
        0 1
        1 1
        2 1
        3 1
        4 2
        6 1
        7 1
        9 2
        10 1

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        """
        Gets format of pitch-class vector.

        ..  container:: example

            >>> vector = abjad.PitchClassVector(
            ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
            ...     item_class=abjad.NumberedPitchClass,
            ...     )

            >>> abjad.f(vector)
            abjad.PitchClassVector(
                {
                    abjad.NumberedPitchClass(0): 1,
                    abjad.NumberedPitchClass(1): 1,
                    abjad.NumberedPitchClass(2): 1,
                    abjad.NumberedPitchClass(3): 1,
                    abjad.NumberedPitchClass(4): 2,
                    abjad.NumberedPitchClass(6): 1,
                    abjad.NumberedPitchClass(7): 1,
                    abjad.NumberedPitchClass(9): 2,
                    abjad.NumberedPitchClass(10): 1,
                    },
                item_class=abjad.NumberedPitchClass,
                )

        Returns string.
        """
        return super().__format__(format_specification=format_specification)

    def __repr__(self):
        """
        Gets interpreter representation of pitch-class vector.

        ..  container:: example

            Gets interpreter representation of pitch-class vector:

            >>> vector = abjad.PitchClassVector(
            ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
            ...     item_class=abjad.NumberedPitchClass,
            ...     )

            >>> vector
            PitchClassVector({0: 1, 1: 1, 2: 1, 3: 1, 4: 2, 6: 1, 7: 1, 9: 2, 10: 1}, item_class=NumberedPitchClass)

        ..  container:: example

            Initializes from interpreter representation of pitch-class vector:


                >>> abjad.PitchClassVector(vector)
                PitchClassVector({0: 1, 1: 1, 2: 1, 3: 1, 4: 2, 6: 1, 7: 1, 9: 2, 10: 1}, item_class=NumberedPitchClass)

        Returns string.
        """
        return super().__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedPitchClass

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedPitchClass

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.PitchClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes pitch-class vector from `selection`.

        Returns pitch-class vector.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



from .Context import Context


class Staff(Context):
    r"""
    Staff.

    ..  container:: example

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'8
                d'8
                e'8
                f'8
            }

    ..  container:: example

        Can initialize from collection of strings:

        >>> staff = abjad.Staff([
        ...     r"\times 9/10 { r8 c'16 c'16 bf'4~ bf'16 r16 }",
        ...     r"\times 9/10 { bf'16 e''16 e''4 ~ e''16 r16 fs''16 af''16 }",
        ...     r"\times 4/5 { a'16 r4 }",
        ...     ])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                {
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 9/10 {
                        r8
                        c'16
                        c'16
                        bf'4
                        ~
                        bf'16
                        r16
                    }
                }
                {
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 9/10 {
                        bf'16
                        e''16
                        e''4
                        ~
                        e''16
                        r16
                        fs''16
                        af''16
                    }
                }
                {
                    \times 4/5 {
                        a'16
                        r4
                    }
                }
            }

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Contexts'

    __slots__ = ()

    _default_lilypond_type = 'Staff'

    ### INITIALIZER ###

    def __init__(
        self,
        components=None,
        lilypond_type='Staff',
        is_simultaneous=None,
        name=None,
        ):
        Context.__init__(
            self,
            components=components,
            lilypond_type=lilypond_type,
            is_simultaneous=is_simultaneous,
            name=name,
            )



import copy
from abjad.utilities.Enumerator import Enumerator
from .Set import Set


class PitchClassSet(Set):
    """
    Pitch-class set.

    ..  container:: example

        Initializes numbered pitch-class set:

        >>> numbered_pitch_class_set = abjad.PitchClassSet(
        ...     items=[-2, -1.5, 6, 7, -1.5, 7],
        ...     item_class=abjad.NumberedPitchClass,
        ...     )
        >>> numbered_pitch_class_set
        PitchClassSet([6, 7, 10, 10.5])

    ..  container:: example

        Initializes named pitch-class set:

        >>> named_pitch_class_set = abjad.PitchClassSet(
        ...     items=['c', 'ef', 'bqs,', 'd'],
        ...     item_class=abjad.NamedPitchClass,
        ...     )
        >>> named_pitch_class_set
        PitchClassSet(['c', 'd', 'ef', 'bqs'])

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        """
        Is true when pitch-class set contains `argument`.

        ..  container:: example

            Initializes numbered pitch-class set:

            >>> set_ = abjad.PitchClassSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitchClass,
            ...     )
            >>> set_
            PitchClassSet([6, 7, 10, 10.5])

            >>> abjad.NamedPitch('fs') in set_
            True

            >>> abjad.NamedPitch('f') in set_
            False

            >>> 6 in set_
            True

            >>> 5 in set_
            False

        Returns true or false.
        """
        return super().__contains__(argument)

    def __hash__(self):
        """
        Hashes pitch-class set.

        Returns integer.
        """
        return super().__hash__()

    def __illustrate__(self):
        r"""
        Illustrates pitch-class set.

        ..  container:: example

            Illustrates numbered segment:

            >>> setting = abjad.PitchClassSet([-2, -1.5, 6, 7, -1.5, 7])
            >>> abjad.show(setting) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = setting.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    <fs' g' bf' bqf'>1
                }

        ..  container:: example

            Illustrates named set:

            >>> items = ['c', 'ef', 'bqs,', 'd']
            >>> setting = abjad.PitchClassSet(
            ...     items=items,
            ...     item_class=abjad.NamedPitchClass,
            ...     )
            >>> abjad.show(setting) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = setting.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    <c' d' ef' bqs'>1
                }

        ..  container:: example

            Returns LilyPond file:

            >>> prototype = abjad.LilyPondFile
            >>> isinstance(setting.__illustrate__(), prototype)
            True

        """
        import abjad
        chord = abjad.Chord(self, abjad.Duration(1))
        voice = abjad.Voice([chord])
        staff = abjad.Staff([voice])
        score = abjad.Score([staff])
        lilypond_file = abjad.LilyPondFile.new(score)
        return lilypond_file

    def __str__(self):
        """
        Gets string representation of pitch-class set.

        ..  container:: example

            Gets string of set sorted at initialization:

            >>> pc_set = abjad.PitchClassSet([6, 7, 10, 10.5])
            >>> str(pc_set)
            'PC{6, 7, 10, 10.5}'

        ..  container:: example

            Gets string of set not sorted at initialization:

            >>> pc_set = abjad.PitchClassSet([10.5, 10, 7, 6])
            >>> str(pc_set)
            'PC{6, 7, 10, 10.5}'

        Returns string.
        """
        import abjad
        items = [str(_) for _ in sorted(self)]
        separator = ' '
        if self.item_class is abjad.NumberedPitchClass:
            separator = ', '
        return 'PC{{{}}}'.format(separator.join(items))

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedPitchClass

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedPitchClass

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.PitchClass

    ### PRIVATE METHODS ###

    @staticmethod
    def _get_most_compact_ordering(candidates):
        import abjad
        widths = []
        for candidate in candidates:
            if candidate[0] < candidate[-1]:
                width = abs(candidate[-1] - candidate[0])
            else:
                width = abs(candidate[-1] + 12 - candidate[0])
            widths.append(width)
        minimum_width = min(widths)
        candidates_ = []
        for candidate, width in zip(candidates, widths):
            if width == minimum_width:
                candidates_.append(candidate)
        candidates = candidates_
        assert 1 <= len(candidates)
        if len(candidates) == 1:
            segment = candidates[0]
            segment = abjad.PitchClassSegment(
                items=segment,
                item_class=abjad.NumberedPitchClass,
                )
            return segment
        for i in range(len(candidates[0]) - 1):
            widths = []
            for candidate in candidates:
                stop = i + 1
                if candidate[0] < candidate[stop]:
                    width = abs(candidate[stop] - candidate[0])
                else:
                    width = abs(candidate[stop] + 12 - candidate[0])
                widths.append(width)
            minimum_width = min(widths)
            candidates_ = []
            for candidate, width in zip(candidates, widths):
                if width == minimum_width:
                    candidates_.append(candidate)
            candidates = candidates_
            if len(candidates) == 1:
                segment = candidates[0]
                segment = abjad.PitchClassSegment(
                    items=segment,
                    item_class=abjad.NumberedPitchClass,
                    )
                return segment
        candidates.sort(key=lambda x: x[0])
        segment = candidates[0]
        segment = abjad.PitchClassSegment(
            items=segment,
            item_class=abjad.NumberedPitchClass,
            )
        return segment

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes pitch-class set from `selection`.

        ..  container:: example

            >>> staff_1 = abjad.Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = abjad.Staff("c4. r8 g2")
            >>> selection = abjad.select((staff_1, staff_2))
            >>> abjad.PitchClassSet.from_selection(selection)
            PitchClassSet(['c', 'd', 'fs', 'g', 'a', 'b'])

        Returns pitch-class set.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        return class_(
            items=pitch_segment,
            item_class=item_class,
            )

    def get_normal_order(self):
        """
        Gets normal order.

        ..  container:: example

            Gets normal order of empty pitch-class set:

            >>> pc_set = abjad.PitchClassSet()
            >>> pc_set.get_normal_order()
            PitchClassSegment([])

        ..  container:: example

            Gets normal order:

            >>> pc_set = abjad.PitchClassSet([0, 1, 10, 11])
            >>> pc_set.get_normal_order()
            PitchClassSegment([10, 11, 0, 1])

        ..  container:: example

            Gets normal order:

            >>> pc_set = abjad.PitchClassSet([2, 8, 9])
            >>> pc_set.get_normal_order()
            PitchClassSegment([8, 9, 2])

        ..  container:: example

            Gets normal order of pitch-class set with degree of symmetry equal
            to 2:

            >>> pc_set = abjad.PitchClassSet([1, 2, 7, 8])
            >>> pc_set.get_normal_order()
            PitchClassSegment([1, 2, 7, 8])

        ..  container:: example

            Gets normal order of pitch-class set with degree of symmetry equal
            to 4:

            >>> pc_set = abjad.PitchClassSet([0, 3, 6, 9])
            >>> pc_set.get_normal_order()
            PitchClassSegment([0, 3, 6, 9])

        Returns pitch-class segment.
        """
        import abjad
        if not len(self):
            return abjad.PitchClassSegment(
                items=None,
                item_class=abjad.NumberedPitchClass,
                )
        pitch_classes = list(self)
        pitch_classes.sort()
        candidates = []
        for i in range(self.cardinality):
            candidate = [abjad.NumberedPitch(_) for _ in pitch_classes]
            candidate = abjad.sequence(candidate).rotate(n=-i)
            candidates.append(candidate)
        return self._get_most_compact_ordering(candidates)

    def get_prime_form(self, transposition_only=False):
        """
        Gets prime form.

        ..  container:: example

            Gets prime form of empty pitch-class set:

            >>> pc_set = abjad.PitchClassSet()
            >>> pc_set.get_prime_form()
            PitchClassSet([])

            >>> pc_set = abjad.PitchClassSet()
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([])

        ..  container:: example

            Gets prime form:

            >>> pc_set = abjad.PitchClassSet([0, 1, 10, 11])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 2, 3])

            >>> pc_set = abjad.PitchClassSet([0, 1, 10, 11])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 1, 2, 3])

        ..  container:: example

            Gets prime form:

            >>> pc_set = abjad.PitchClassSet([2, 8, 9])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 6])

            >>> pc_set = abjad.PitchClassSet([2, 8, 9])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 1, 6])

        ..  container:: example

            Gets prime form of pitch-class set with degree of symmetry equal to
            2:

            >>> pc_set = abjad.PitchClassSet([1, 2, 7, 8])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 6, 7])

            >>> pc_set = abjad.PitchClassSet([1, 2, 7, 8])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 1, 6, 7])

        ..  container:: example

            Gets prime form of pitch-class set with degree of symmetry equal to
            4:

            >>> pc_set = abjad.PitchClassSet([0, 3, 6, 9])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 3, 6, 9])

            >>> pc_set = abjad.PitchClassSet([0, 3, 6, 9])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 3, 6, 9])

        ..  container:: example

            Gets prime form of pitch-class that is not inversion-equivalent:

            >>> pc_set = abjad.PitchClassSet([0, 4, 6, 7])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 3, 7])

            >>> pc_set = abjad.PitchClassSet([0, 4, 6, 7])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 4, 6, 7])

        ..  container:: example

            Gets prime form of inversionally nonequivalent pitch-class set:

            >>> pc_set = abjad.PitchClassSet([0, 4, 7])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 3, 7])

            >>> pc_set = abjad.PitchClassSet([0, 4, 7])
            >>> pc_set.get_prime_form(transposition_only=True)
            PitchClassSet([0, 4, 7])

        ..  container:: example

            REGRESSION:

            >>> pc_set = abjad.PitchClassSet([0, 1, 2, 5, 8, 9])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 2, 5, 6, 9])

            REGRESSION:

            >>> pc_set = abjad.PitchClassSet([0, 1, 2, 3, 6, 7])
            >>> pc_set.get_prime_form()
            PitchClassSet([0, 1, 2, 3, 6, 7])

        Returns new pitch-class set.
        """
        import abjad
        if not len(self):
            return copy.copy(self)
        normal_order = self.get_normal_order()
        if not transposition_only:
            normal_orders = [normal_order]
            inversion = self.invert()
            normal_order = inversion.get_normal_order()
            normal_orders.append(normal_order)
            normal_orders = [_._transpose_to_zero() for _ in normal_orders]
            assert len(normal_orders) == 2
            for left_pc, right_pc in zip(*normal_orders):
                if left_pc == right_pc:
                    continue
                if left_pc < right_pc:
                    normal_order = normal_orders[0]
                    break
                if right_pc < left_pc:
                    normal_order = normal_orders[-1]
                    break
        pcs = [_.number for _ in normal_order]
        first_pc = pcs[0]
        pcs = [pc - first_pc for pc in pcs]
        prime_form = type(self)(
            items=pcs,
            item_class=abjad.NumberedPitchClass,
            )
        return prime_form

    def invert(self, axis=None):
        """
        Inverts pitch-class set.

        ..  container:: example

            >>> abjad.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     ).invert()
            PitchClassSet([1.5, 2, 5, 6])

        Returns numbered pitch-class set.
        """
        return type(self)([pc.invert(axis=axis) for pc in self])

    def is_transposed_subset(self, pcset):
        """
        Is true when pitch-class set is transposed subset of `pcset`.

        ..  container:: example

            >>> pitch_class_set_1 = abjad.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_set_2 = abjad.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7, 7.5, 8],
            ...     )

            >>> pitch_class_set_1.is_transposed_subset(pitch_class_set_2)
            True

        Returns true or false.
        """
        for n in range(12):
            if self.transpose(n).issubset(pcset):
                return True
        return False

    def is_transposed_superset(self, pcset):
        """
        Is true when pitch-class set is transposed superset of `pcset`.

        ..  container:: example

            >>> pitch_class_set_1 = abjad.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_set_2 = abjad.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7, 7.5, 8],
            ...     )

            >>> pitch_class_set_2.is_transposed_superset(pitch_class_set_1)
            True

        Returns true or false.
        """
        for n in range(12):
            if self.transpose(n).issuperset(pcset):
                return True
        return False

    def multiply(self, n):
        """
        Multiplies pitch-class set by `n`.

        ..  container:: example

            >>> abjad.PitchClassSet(
            ...     [-2, -1.5, 6, 7, -1.5, 7],
            ...     ).multiply(5)
            PitchClassSet([2, 4.5, 6, 11])

        Returns new pitch-class set.
        """
        import abjad
        items = (pitch_class.multiply(n) for pitch_class in self)
        return abjad.new(self, items=items)

    def order_by(self, segment):
        """
        Orders pitch-class set by pitch-class `segment`.

        ..  container:: example

            >>> set_ = abjad.PitchClassSet(['c', 'e', 'b'])
            >>> segment = abjad.PitchClassSegment(['e', 'a', 'f'])
            >>> set_.order_by(segment)
            PitchClassSegment("b e c")

        Returns pitch-class segment.
        """
        import abjad
        if not len(self) == len(segment):
            message = 'set and segment must be on equal length.'
            raise ValueError(message)
        enumerator = Enumerator(self)
        for pitch_classes in enumerator.yield_permutations():
            candidate = abjad.PitchClassSegment(pitch_classes)
            if candidate._is_equivalent_under_transposition(segment):
                return candidate
        message = '{!s} can not order by {!s}.'
        message = message.format(self, segment)
        raise ValueError(message)

    def transpose(self, n=0):
        """
        Transposes all pitch-classes in pitch-class set by index `n`.

        ..  container:: example

            >>> set_ = abjad.PitchClassSet(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NumberedPitchClass,
            ...     )

            >>> for n in range(12):
            ...     print(n, set_.transpose(n))
            ...
            0 PC{6, 7, 10, 10.5}
            1 PC{7, 8, 11, 11.5}
            2 PC{0, 0.5, 8, 9}
            3 PC{1, 1.5, 9, 10}
            4 PC{2, 2.5, 10, 11}
            5 PC{0, 3, 3.5, 11}
            6 PC{0, 1, 4, 4.5}
            7 PC{1, 2, 5, 5.5}
            8 PC{2, 3, 6, 6.5}
            9 PC{3, 4, 7, 7.5}
            10 PC{4, 5, 8, 8.5}
            11 PC{5, 6, 9, 9.5}

        Returns new pitch-class set.
        """
        import abjad
        items = (pitch_class + n for pitch_class in self)
        return abjad.new(self, items=items)



from abjad.system.AbjadObject import AbjadObject


class StorageFormatSpecification(AbjadObject):
    """
    Storage format specification.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Storage formatting'

    __slots__ = (
        '_repr_text',
        '_include_abjad_namespace',
        '_instance',
        '_is_bracketed',
        '_is_indented',
        '_keyword_argument_names',
        '_positional_argument_values',
        '_storage_format_text',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        instance=None,
        repr_text=None,
        include_abjad_namespace=None,
        is_bracketed=False,
        is_indented=True,
        keyword_argument_names=None,
        positional_argument_values=None,
        storage_format_text=None,
        ):
        self._instance = instance

        if repr_text is not None:
            repr_text = str(repr_text)
        self._repr_text = repr_text

        self._include_abjad_namespace = bool(include_abjad_namespace)
        self._is_bracketed = bool(is_bracketed)
        self._is_indented = bool(is_indented)

        if keyword_argument_names is not None:
            keyword_argument_names = tuple(keyword_argument_names)
        self._keyword_argument_names = keyword_argument_names

        if positional_argument_values is not None:
            positional_argument_values = tuple(positional_argument_values)
        self._positional_argument_values = positional_argument_values

        if storage_format_text is not None:
            storage_format_text = str(storage_format_text)
        self._storage_format_text = storage_format_text

    ### PUBLIC PROPERTIES ###

    @property
    def include_abjad_namespace(self):
        """
        Is true when storage specification includes Abjad namespace.

        Returns true or false.
        """
        return self._include_abjad_namespace

    @property
    def instance(self):
        """
        Gets instance of storage specification.

        Returns string.
        """
        return self._instance

    @property
    def is_bracketed(self):
        """
        Is true when storage specification is bracketed.

        Returns true or false.
        """
        return self._is_bracketed

    @property
    def is_indented(self):
        """
        Is true when storage format is indented.

        Returns true or false.
        """
        return self._is_indented

    @property
    def keyword_argument_names(self):
        """
        Gets keyword argument names of storage format.

        Returns tuple.
        """
        return self._keyword_argument_names

    @property
    def positional_argument_values(self):
        """
        Gets positional argument values.

        Returns tuple.
        """
        return self._positional_argument_values

    @property
    def repr_text(self):
        """
        Gets interpreter representation of storage specification.

        Returns string.
        """
        return self._repr_text

    @property
    def storage_format_text(self):
        """
        Gets storage format text.

        Returns tuple.
        """
        return self._storage_format_text



import abjad
import definition
import ide
import sys
import traceback


if __name__ == '__main__':

    directory, lilypond_file = ide.Path(__file__).parent, None

    try:
        material = getattr(definition, directory.name)
    except ImportError:
        traceback.print_exc()
        sys.exit(1)

    try:
        with abjad.Timer() as timer:
            if getattr(definition, '__illustrate__', None):
                __illustrate__ = getattr(definition, '__illustrate__')
                lilypond_file = __illustrate__(material)
            elif hasattr(material, '__illustrate__'):
                lilypond_file = material.__illustrate__()
            else:
                print(f'No illustrate method ...')
        count = int(timer.elapsed_time)
        counter = abjad.String('second').pluralize(count)
        message = f'Abjad runtime {count} {counter} ...'
        print(message)
    except:
        traceback.print_exc()
        sys.exit(1)

    if lilypond_file is None:
        sys.exit(0)

    try:
        ly = directory('illustration.ly')
        with abjad.Timer() as timer:
            abjad.persist(lilypond_file).as_ly(ly, strict=89)
        count = int(timer.elapsed_time)
        counter = abjad.String('second').pluralize(count)
        message = f'LilyPond runtime {count} {counter} ...'
        print(message)
    except:
        traceback.print_exc()
        sys.exit(1)

    sys.exit(0)



from abjad.system.AbjadObject import AbjadObject


class Line(AbjadObject):
    r"""
    Line in a LilyPond file.

    ..  container:: example

        >>> string = '    %@%  \with-color %! MEASURE_NUMBER_MARKUP:SM31'
        >>> abjad.Line(string)
        Line(string='    %@%  \\with-color %! MEASURE_NUMBER_MARKUP:SM31')

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Segment-makers'

    __slots__ = (
        '_string',
        )

    ### INITIALIZER ###

    def __init__(self, string=''):
        assert isinstance(string, str), repr(string)
        self._string = string

    ### SPECIAL METHODS ###

    def __str__(self):
        r"""
        Gets string representation of line.

        ..  container:: example

            >>> string = '    %@%  \with-color %! MEASURE_NUMBER_MARKUP:SM31'
            >>> str(abjad.Line(string))
            '    %@%  \\with-color %! MEASURE_NUMBER_MARKUP:SM31'

        Returns string.
        """
        return self.string

    ### PUBLIC PROPERTIES ###

    @property
    def string(self):
        r"""
        Gets string.

        ..  container:: example

            >>> string = '    %@%  \with-color %! MEASURE_NUMBER_MARKUP:SM31'
            >>> abjad.Line(string).string
            '    %@%  \\with-color %! MEASURE_NUMBER_MARKUP:SM31'

        Returns string.
        """
        return self._string

    ### PUBLIC METHODS ###

    def get_tags(self):
        """
        Gets tags.

        ..  container:: example

            >>> string = '    %@%  \with-color %! MEASURE_NUMBER_MARKUP:SM31'
            >>> abjad.Line(string).get_tags()
            ['MEASURE_NUMBER_MARKUP', 'SM31']

        Returns list of zero or more strings.
        """
        tags = []
        if ' %! ' in self.string:
            index = self.string.find('%! ')
            string = self.string[index+2:].strip()
            parts = string.split()
            tags.extend(parts[0].split(':'))
        return tags

    def is_active(self):
        r"""
        Is true when line is active.

        ..  container:: example

            >>> string = '              \\clef "treble" %! EXPLICT_CLEF'
            >>> abjad.Line(string).is_active()
            True

            >>> string = '          %@% \\clef "treble" %! EXPLICT_CLEF'
            >>> abjad.Line(string).is_active()
            False

            >>> string = '          %%% \\clef "treble" %! EXPLICT_CLEF'
            >>> abjad.Line(string).is_active()
            False

        Returns true or false.
        """
        return not self.is_deactivated()

    def is_deactivated(self):
        r"""
        Is true when line is deactivated.

        ..  container:: example

            >>> string = '              \\clef "treble" %! EXPLICT_CLEF'
            >>> abjad.Line(string).is_deactivated()
            False

            >>> string = '          %@% \\clef "treble" %! EXPLICT_CLEF'
            >>> abjad.Line(string).is_deactivated()
            True

            >>> string = '          %%% \\clef "treble" %! EXPLICT_CLEF'
            >>> abjad.Line(string).is_deactivated()
            True

        Returns true or false.
        """
        string = self.string.strip()
        if string.startswith('%@%'):
            return True
        if string.startswith('%%%'):
            return True
        return False

    def match(self, predicate):
        r"""
        Is true when ``predicate`` matches tags.

        ..  container:: example

            >>> string = '    %@%  \with-color %! MEASURE_NUMBER_MARKUP:SM31'
            >>> line = abjad.Line(string)

        ..  container:: example

            Strings:

            >>> line.match('MEASURE_NUMBER_MARKUP')
            True

            >>> line.match('SM31')
            True

            >>> line.match(['MEASURE_NUMBER_MARKUP', 'SM31'])
            False

            >>> line.match('%@%')
            False

            >>> line.match('with-color')
            False

            >>> line.match('%!')
            False

        ..  container:: example

            Lambdas:

            >>> line.match(lambda x: any(_ for _ in x if _.startswith('M')))
            True

            >>> line.match(lambda x: any(_ for _ in x if _.startswith('S')))
            True

            >>> line.match(lambda x: any(_ for _ in x if _[0] in 'SM'))
            True

        ..  container:: example

            Functions:

            >>> def predicate(tags):
            ...     if 'SM31' in tags and 'MEASURE_NUMBER_MARKUP' in tags:
            ...         return True
            ...     else:
            ...         return False

            >>> line.match(predicate)
            True

            >>> def predicate(tags):
            ...     if 'SM31' in tags and 'MEASURE_NUMBER_MARKUP' not in tags:
            ...         return True
            ...     else:
            ...         return False

            >>> line.match(predicate)
            False

        Returns true or false.
        """
        tags = self.get_tags()
        if not tags:
            return False
        if predicate in tags:
            return True
        if not callable(predicate):
            return False
        return predicate(tags)



import typing
from abjad.system.AbjadObject import AbjadObject


class SlotContributions(AbjadObject):
    """
    Slot contributions.
    """

    __documentation_section__ = 'LilyPond formatting'

    __slots__ = (
        '_articulations',
        '_commands',
        '_comments',
        '_indicators',
        '_leaks',
        '_markup',
        '_spanners',
        '_spanner_starts',
        '_spanner_stops',
        '_stem_tremolos',
        '_trill_spanner_starts',
        )

    ### INITIALIZER ###

    def __init__(self) -> None:
        self._articulations: typing.List[str] = []
        self._commands: typing.List[str] = []
        self._comments: typing.List[str] = []
        self._indicators: typing.List[str] = []
        self._leaks: typing.List[str] = []
        self._markup: typing.List[str] = []
        self._spanners: typing.List[str] = []
        self._spanner_starts: typing.List[str] = []
        self._spanner_stops: typing.List[str] = []
        self._stem_tremolos: typing.List[str] = []
        self._trill_spanner_starts: typing.List[str] = []

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        import abjad
        names = [
            'articulations',
            'commands',
            'comments',
            'indicators',
            'markup',
            'spanners',
            'spanner_starts',
            'spanner_stops',
            'stem_tremolos',
            'trill_spanner_starts',
            ]
        names = [_ for _ in names if getattr(self, _)]
        return abjad.FormatSpecification(
            client=self,
            storage_format_kwargs_names=names,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def articulations(self) -> typing.List[str]:
        """
        Gets articulations.
        """
        return self._articulations

    @property
    def commands(self) -> typing.List[str]:
        """
        Gets commands.
        """
        return self._commands

    @property
    def comments(self) -> typing.List[str]:
        """
        Gets comments.
        """
        return self._comments

    @property
    def has_contributions(self) -> bool:
        """
        Is true when has contributions.
        """
        contribution_categories = (
            'articulations',
            'commands',
            'comments',
            'indicators',
            'leaks',
            'markup',
            'spanners',
            'spanner_starts',
            'spanner_stops',
            'stem_tremolos',
            'trill_spanner_starts',
            )
        return any(getattr(self, contribution_category)
            for contribution_category in contribution_categories)

    @property
    def indicators(self) -> typing.List[str]:
        """
        Gets indicators.
        """
        return self._indicators

    @property
    def leaks(self) -> typing.List[str]:
        """
        Gets leaks.
        """
        return self._leaks

    @property
    def markup(self) -> typing.List[str]:
        """
        Gets markup.
        """
        return self._markup

    @property
    def spanner_starts(self) -> typing.List[str]:
        """
        Gets spanner starts.
        """
        return self._spanner_starts

    @property
    def spanner_stops(self) -> typing.List[str]:
        """
        Gets spanner stops.
        """
        return self._spanner_stops

    @property
    def spanners(self) -> typing.List[str]:
        """
        Gets spanners.
        """
        return self._spanners

    @property
    def stem_tremolos(self) -> typing.List[str]:
        """
        Gets stem tremolos.
        """
        return self._stem_tremolos

    @property
    def trill_spanner_starts(self) -> typing.List[str]:
        """
        Gets trill spanner starts.
        """
        return self._trill_spanner_starts

    ### PUBLIC METHODS ###

    def get(self, identifier):
        """
        Gets ``identifier``.
        """
        return getattr(self, identifier)

    def tag(self, tag, deactivate=None):
        """
        Tags contributions.
        """
        import abjad
        self._articulations = abjad.LilyPondFormatManager.tag(
            self.articulations,
            tag,
            deactivate,
            )
        self._commands = abjad.LilyPondFormatManager.tag(
            self.commands,
            tag,
            deactivate,
            )
        self._comments = abjad.LilyPondFormatManager.tag(
            self.comments,
            tag,
            deactivate,
            )
        self._indicators = abjad.LilyPondFormatManager.tag(
            self.indicators,
            tag,
            deactivate,
            )
        self._leaks = abjad.LilyPondFormatManager.tag(
            self.leaks,
            tag,
            deactivate,
            )
        self._markup = abjad.LilyPondFormatManager.tag(
            self.markup,
            tag,
            deactivate,
            )
        self._spanners = abjad.LilyPondFormatManager.tag(
            self.spanners,
            tag,
            deactivate,
            )
        strings = []
        # make sure each line of multiline markup is tagged
        for string in self.spanner_starts:
            strings.extend(string.split('\n'))
        self._spanner_starts = abjad.LilyPondFormatManager.tag(
            strings,
            tag,
            deactivate,
            )
        self._spanner_stops = abjad.LilyPondFormatManager.tag(
            self.spanner_stops,
            tag,
            deactivate,
            )
        self._stem_tremolos = abjad.LilyPondFormatManager.tag(
            self.stem_tremolos,
            tag,
            deactivate,
            )

    def update(self, slot_contributions):
        """
        Updates contributions.
        """
        assert isinstance(slot_contributions, type(self))
        self.articulations.extend(slot_contributions.articulations)
        self.commands.extend(slot_contributions.commands)
        self.comments.extend(slot_contributions.comments)
        self.indicators.extend(slot_contributions.indicators)
        self.leaks.extend(slot_contributions.leaks)
        self.markup.extend(slot_contributions.markup)
        self.spanners.extend(slot_contributions.spanners)
        self.spanner_starts.extend(slot_contributions.spanner_starts)
        self.spanner_stops.extend(slot_contributions.spanner_stops)
        self.stem_tremolos.extend(slot_contributions.stem_tremolos)
        self.trill_spanner_starts.extend(
            slot_contributions.trill_spanner_starts
            )



import typing
from abjad.system.AbjadObject import AbjadObject
from abjad.system.Tag import Tag


class Momento(AbjadObject):
    """
    Momento.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_context',
        '_edition',
        '_manifest',
        '_prototype',
        '_value',
        )

    ### INITIALIZER ###

    def __init__(
        self, 
        context: str = None,
        edition: typing.Union[str, Tag] = None,
        manifest: str = None,
        prototype: str = None,
        value: typing.Any = None,
        ) -> None:
        if context is not None:
            assert isinstance(context, str), repr(context)
        self._context = context
        edition_ = None
        if edition is not None:
            edition_ = Tag(edition)
        self._edition = edition_
        if manifest is not None:
            assert isinstance(manifest, str), repr(manifest)
            assert prototype is None
        self._manifest = manifest
        if prototype is not None:
            assert isinstance(prototype, str), repr(prototype)
            assert manifest is None
        self._prototype = prototype
        if value is not None:
            if not isinstance(value, (int, str, dict)):
                assert type(value).__name__ == 'PersistentOverride', repr(value)
        self._value = value

    ### PUBLIC PROPERTIES ###

    @property
    def context(self) -> typing.Optional[str]:
        """
        Gets (name of local) context.
        """
        return self._context

    @property
    def edition(self) -> typing.Optional[Tag]:
        """
        Gets edition.
        """
        return self._edition

    @property
    def manifest(self) -> typing.Optional[str]:
        """
        Gets manifest.
        """
        return self._manifest

    @property
    def prototype(self) -> typing.Optional[str]:
        """
        Gets prototype.
        """
        return self._prototype

    @property
    def value(self) -> typing.Union[int, str]:
        """
        Gets value.
        """
        return self._value



import importlib
import os
import subprocess
from abjad import system
from abjad.system.AbjadValueObject import AbjadValueObject


class PackageGitCommitToken(AbjadValueObject):
    """
    A Python package git commit token.

    ..  container:: example

        >>> token = abjad.PackageGitCommitToken('abjad')
        >>> token
        PackageGitCommitToken(package_name='abjad')

        >>> print(format(token))  # doctest: +SKIP
        package "abjad" @ b6a48a7 [implement-lpf-git-token] (2016-02-02 13:36:25)

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_package_name',
        )

    ### INITIALIZER ###

    def __init__(self, package_name=None):
        self._package_name = package_name

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        """
        Formats package git commit token.

        ..  container:: example

            >>> token = abjad.PackageGitCommitToken('abjad')
            >>> print(format(token)) # doctest: +SKIP
            package "abjad" @ b6a48a7 [implement-lpf-git-token] (2016-02-02 13:36:25)

        Return string.
        """
        if not self.package_name:
            return ''
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return system.StorageFormatManager(self).get_storage_format()
        return str(self)

    ### PRIVATE METHODS ###

    def _get_commit_timestamp(self, commit_hash):
        command = 'git show -s --format=%ci {}'.format(commit_hash)
        return self._run_command(command)

    def _get_git_branch(self):
        command = 'git rev-parse --abbrev-ref HEAD'
        return self._run_command(command)

    def _get_git_hash(self):
        command = 'git rev-parse HEAD'
        return self._run_command(command)

    def _get_lilypond_format(self):
        path = self._get_package_path()
        with system.TemporaryDirectoryChange(path):
            git_branch = self._get_git_branch()
            git_hash = self._get_git_hash()
            timestamp = self._get_commit_timestamp(git_hash)
            #print(git_branch, git_hash, timestamp)
        date, time, _ = timestamp.split()
        return 'package "{}" @ {} [{}] ({} {})'.format(
            self._package_name,
            git_hash[:7],
            git_branch,
            date,
            time,
            )

    def _get_package_path(self):
        module = importlib.import_module(self._package_name)
        path = module.__path__[0]
        if not os.path.isdir(path):
            path = os.path.dirname(path)
        path = os.path.abspath(path)
        return path

    def _run_command(self, command):
        process = subprocess.Popen(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            )
        process.wait()
        if process.returncode:
            return None
        result = process.stdout.read().splitlines()[0]
        result = result.decode('utf-8')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def package_name(self):
        """
        Gets package name of package git commit token.

        ..  container:: example

            >>> token = abjad.PackageGitCommitToken('abjad')
            >>> token.package_name
            'abjad'

        Returns string.
        """
        return self._package_name
